<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions"
  xmlns:local="ns:local" exclude-result-prefixes="#all">
  <!--* XSL om de RP templates voor de meervoudige interactietypes te genereren -->
  <!-- -->
  <xsl:output method="xml" indent="yes" encoding="UTF-8"/>
  <!-- -->
  <xsl:variable name="FilenamePrefix" as="xs:string" select="'RPTEMPLATE_'"/>
  <xsl:variable name="ResponseIdentifierPrefix" as="xs:string" select="'RESPONSE_'"/>
  <xsl:variable name="TemplateCount" as="xs:integer" select="10"/>
  <!-- -->
  <xsl:template match="/">
    <CreatedRPTemplates timestamp="{current-dateTime()}">
      <xsl:call-template name="Create_GF_xx">
        <xsl:with-param name="WithFeedback" select="false()"/>
      </xsl:call-template>
      <xsl:call-template name="Create_GF_xx">
        <xsl:with-param name="WithFeedback" select="true()"/>
      </xsl:call-template>
      <!-- -->
      <xsl:call-template name="Create_SCORE_xx">
        <xsl:with-param name="WithFeedback" select="false()"/>
      </xsl:call-template>
      <xsl:call-template name="Create_SCORE_xx">
        <xsl:with-param name="WithFeedback" select="true()"/>
      </xsl:call-template>
      <!-- -->
      <xsl:call-template name="Create_SCORE_xx">
        <xsl:with-param name="WithFeedback" select="false()"/>
        <xsl:with-param name="PointScore" select="true()"/>
      </xsl:call-template>
      <xsl:call-template name="Create_SCORE_xx">
        <xsl:with-param name="WithFeedback" select="true()"/>
        <xsl:with-param name="PointScore" select="true()"/>
      </xsl:call-template>
      <!-- -->
    </CreatedRPTemplates>
  </xsl:template>
  <!-- -->
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <!-- -->
  <xsl:template name="Create_GF_xx">
    <xsl:param name="WithFeedback" as="xs:boolean" required="yes"/>
    <!-- -->
    <xsl:variable name="TemplateName" as="xs:string"
      select="concat('GF', if ($WithFeedback) then '_FB1' else '')"/>
    <xsl:for-each select="1 to $TemplateCount">
      <xsl:variable name="Index" as="xs:integer" select="."/>
      <xsl:variable name="Filename" as="xs:string"
        select="concat($FilenamePrefix, $TemplateName, '_', local:Index2Suffix($Index), '.xml')"/>
      <!-- -->
      <xsl:result-document href="{$Filename}" method="xml" encoding="UTF-8" indent="yes">
        <responseProcessing xmlns="http://www.imsglobal.org/xsd/imsqti_v2p1">
          <xsl:comment>* NLQTI Response Processing Template - <xsl:value-of select="$TemplateName"/> for plural interaction type - Correct/Incorrect situation <xsl:value-of select="if ($WithFeedback) then '+ Feedback' else ''"/> - <xsl:value-of select="$Index"/> interaction(s) </xsl:comment>
          <xsl:comment> == File: <xsl:value-of select="$Filename"/> == </xsl:comment>
          <!-- -->
          <responseCondition>
            <responseIf>
              <and>
                <xsl:for-each select="1 to $Index">
                  <match>
                    <variable identifier="{local:ResponseIdentifier(.)}"/>
                    <correct identifier="{local:ResponseIdentifier(.)}"/>
                  </match>
                </xsl:for-each>
              </and>
              <setOutcomeValue identifier="SCORE">
                <baseValue baseType="integer">1.0</baseValue>
              </setOutcomeValue>
              <xsl:if test="$WithFeedback">
                <setOutcomeValue identifier="FEEDBACK">
                  <baseValue baseType="identifier">ANSWER_CORRECT</baseValue>
                </setOutcomeValue>
              </xsl:if>
            </responseIf>
            <responseElse>
              <setOutcomeValue identifier="SCORE">
                <baseValue baseType="integer">0.0</baseValue>
              </setOutcomeValue>
              <xsl:if test="$WithFeedback">
                <setOutcomeValue identifier="FEEDBACK">
                  <baseValue baseType="identifier">FAILURE</baseValue>
                </setOutcomeValue>
              </xsl:if>
            </responseElse>
          </responseCondition>
          <!-- -->
        </responseProcessing>
      </xsl:result-document>
      <!-- -->
      <Created type="{$TemplateName}" name="{$Filename}"/>
    </xsl:for-each>
  </xsl:template>
  <!-- -->
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <!-- -->
  <xsl:template name="Create_SCORE_xx">
    <xsl:param name="WithFeedback" as="xs:boolean" required="yes"/>
    <xsl:param name="PointScore" as="xs:boolean" required="no" select="false()"/>
    <!-- -->
    <xsl:variable name="TemplateName" as="xs:string"
      select="concat(
        if ($PointScore) then 'POINT' else 'SCORE', 
        if ($WithFeedback) then '_FB1' else '')"/>
    <xsl:for-each select="1 to $TemplateCount">
      <xsl:variable name="Index" as="xs:integer" select="."/>
      <xsl:variable name="Filename" as="xs:string"
        select="concat($FilenamePrefix, $TemplateName, '_', local:Index2Suffix($Index), '.xml')"/>
      <!-- -->
      <xsl:result-document href="{$Filename}" method="xml" encoding="UTF-8" indent="yes">
        <responseProcessing xmlns="http://www.imsglobal.org/xsd/imsqti_v2p1">
          <xsl:comment>* NLQTI Response Processing Template - <xsl:value-of select="$TemplateName"/> for plural interaction type <xsl:value-of select="if ($PointScore) then '(graphical)' else ''"/> - Scoring situation <xsl:value-of select="if ($WithFeedback) then '+ Feedback' else ''"/> - <xsl:value-of select="$Index"/> interaction(s) </xsl:comment>
          <xsl:comment> == File: <xsl:value-of select="$Filename"/> == </xsl:comment>
          <!-- -->
          <setOutcomeValue identifier="SCORE">
            <sum>
              <xsl:for-each select="1 to $Index">
                <xsl:choose>
                  <xsl:when test="$PointScore">
                    <mapResponsePoint identifier="{local:ResponseIdentifier(.)}"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <mapResponse identifier="{local:ResponseIdentifier(.)}"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:for-each>
            </sum>
          </setOutcomeValue>
          <xsl:comment> == Limit SCORE: == </xsl:comment>
          <responseCondition>
            <responseIf>
              <gt>
                <variable identifier="SCORE"/>
                <baseValue baseType="float">1.0</baseValue>
              </gt>
              <setOutcomeValue identifier="SCORE">
                <baseValue baseType="float">1.0</baseValue>
              </setOutcomeValue>
            </responseIf>
            <responseElseIf>
              <lt>
                <variable identifier="SCORE"/>
                <baseValue baseType="float">0.0</baseValue>
              </lt>
              <setOutcomeValue identifier="SCORE">
                <baseValue baseType="float">0.0</baseValue>
              </setOutcomeValue>
            </responseElseIf>
          </responseCondition>
          <!-- -->
          <xsl:if test="$WithFeedback">
            <xsl:comment> == Feedback: == </xsl:comment>
            <responseCondition>
              <responseIf>
                <gte>
                  <variable identifier="SCORE"/>
                  <variable identifier="FEEDBACK_THRESHOLD"/>
                </gte>
                <setOutcomeValue identifier="FEEDBACK">
                  <baseValue baseType="identifier">ANSWER_CORRECT</baseValue>
                </setOutcomeValue>
              </responseIf>
              <responseElse>
                <setOutcomeValue identifier="FEEDBACK">
                  <baseValue baseType="identifier">FAILURE</baseValue>
                </setOutcomeValue>
              </responseElse>
            </responseCondition>
          </xsl:if>
          <!-- -->
        </responseProcessing>
      </xsl:result-document>
      <!-- -->
      <Created type="{$TemplateName}" name="{$Filename}"/>
    </xsl:for-each>
  </xsl:template>
  <!-- -->
  <!-- ================================================================== -->
  <!-- HELPERS: -->
  <!-- -->
  <xsl:function name="local:Index2Suffix" as="xs:string">
    <xsl:param name="Index" as="xs:integer"/>
    <!-- -->
    <!-- We assume here that the index is always <= 99 -->
    <xsl:choose>
      <xsl:when test="$Index lt 10">
        <xsl:sequence select="concat('0', string($Index))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="string($Index)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  <!-- -->
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <!-- -->
  <xsl:function name="local:ResponseIdentifier" as="xs:string">
    <xsl:param name="Index" as="xs:integer"/>
    <!-- -->
    <xsl:sequence select="concat($ResponseIdentifierPrefix, local:Index2Suffix($Index))"/>
  </xsl:function>
  <!-- -->
</xsl:stylesheet>
