<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron" queryBinding="xslt2">
  <!-- ================================================================== -->
  <!--* ISO Schematron to verify QTI items according to the NLQTI profile. -->
  <!--
    For full validation: Validate first against the (adapted) schema and after that against this 
    file.  
  
    SVN: $Id: imsqti_v2p1_NLQTI_v1p1_items.sch 1317 2013-08-08 11:56:56Z erik $ 
  -->
  <!-- ================================================================== -->
  <!-- -->
  <ns uri="http://www.imsglobal.org/xsd/imsqti_v2p1" prefix="qti"/>
  <ns uri="http://www.w3.org/2001/XMLSchema" prefix="xs"/>
  <!-- -->
  <let name="InteractionCountWithoutMedia"
    value="count(/*/qti:itemBody//qti:*[ends-with(local-name(), 'Interaction') and (local-name() ne 'mediaInteraction')])"/>
  <let name="InteractionCount"
    value="count(/*/qti:itemBody//qti:*[ends-with(local-name(), 'Interaction')])"/>
  <let name="ItemWithInteraction" value="$InteractionCount gt 0"/>
  <let name="HasFeedback" value="exists(/*/qti:modalFeedback)"/>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECKS FOR NON-INTERACTIVE ITEMS: -->
  <!-- -->
  <pattern>
    <!-- An item without interaction must only have a qti:itemBody element (checked by the schema) 
      but no other top level elements (checked here): -->
    <rule context="/qti:assessmentItem[not($ItemWithInteraction)]">
      <assert test="not(exists(qti:responseDeclaration))">Item without interaction:
        responseDeclaration elements not allowed</assert>
      <assert test="not(exists(qti:outcomeDeclaration))">Item without interaction:
        outcomeDeclaration elements not allowed</assert>
      <assert test="exists(qti:itemBody)">Item without interaction: itemBody element is
        mandatory</assert>
      <assert test="not(exists(qti:responseProcessing))">Item without interaction:
        responseProcessing not allowed</assert>
      <assert test="not(exists(qti:modalFeedback))">Item without interaction: modalFeedback not
        allowed</assert>
    </rule>
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECK THE OVERALL STRUCTURE: -->
  <!-- -->
  <pattern>
    <!-- -->
    <rule context="/qti:assessmentItem[$ItemWithInteraction]">
      <assert test="count(qti:responseDeclaration) eq $InteractionCount">The number of
        responseDeclaration elements (<value-of select="count(qti:responseDeclaration)"/>) must
        equal the number of interactions (<value-of select="$InteractionCount"/>)</assert>
      <assert
        test="every $od in qti:outcomeDeclaration satisfies $od/@identifier = ('SCORE', 'FEEDBACK', 'FEEDBACK_THRESHOLD')"
        >outcomeDeclaration element(s) found with an invalid identifier (allowed: "SCORE",
        "FEEDBACK", "FEEDBACK_THRESHOLD")</assert>
    </rule>
    <!-- -->
  </pattern>
  <!-- -->
  <!-- Checks specific to feedback situations:-->
  <pattern>
    <!-- -->
    <rule context="/qti:assessmentItem[$ItemWithInteraction][$HasFeedback]">
      <assert test="qti:outcomeDeclaration[@identifier eq 'FEEDBACK']">Found an item with feedback
        without an outcomeDeclaration "FEEDBACK"</assert>
      <assert
        test="every $mf in qti:modalFeedback satisfies $mf/@identifier = ('FAILURE', 'ANSWER_CORRECT')"
        >modalFeedback element(s) element(s) found with an invalid identifier (allowed: "FAILURE",
        "ANSWER_CORRECT")</assert>
      <assert test="qti:modalFeedback[@identifier eq 'FAILURE']">modalFeedback "FAILURE" not
        found</assert>
      <assert
        test="not(qti:modalFeedback[@identifier eq 'ANSWER_CORRECT']) or 
          (qti:modalFeedback[@identifier eq 'ANSWER_CORRECT'] and qti:modalFeedback[@identifier eq 'FAILURE'])"
        >modalFeedback "ANSWER_CORRECT" present without modalFeedback "FAILURE"</assert>
    </rule>
    <!-- -->
    <rule context="qti:assessmentItem[$ItemWithInteraction][not($HasFeedback)]">
      <assert test="not(qti:outcomeDeclaration[@identifier eq 'FEEDBACK'])">outcomeDeclaration
        "FEEDBACK" found in an item without feedback</assert>
      <assert test="not(qti:outcomeDeclaration[@identifier eq 'FEEDBACK_THRESHOLD'])"
        >outcomeDeclaration "FEEDBACK_THRESHOLD" found in an item without feedback</assert>
      <!-- -->
    </rule>
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECK DETAILS FOR THE OUTCOME DECLARATIONS: -->
  <!-- -->
  <pattern>
    <!-- -->
    <rule context="qti:outcomeDeclaration[$ItemWithInteraction][@identifier eq 'SCORE']">
      <assert test="@baseType eq 'float'">outcomeDeclaration "SCORE" must have baseType
        "float"</assert>
      <assert test="@normalMinimum castable as xs:float and xs:float(@normalMinimum) eq 0.0"
        >outcomeDeclaration "SCORE" must have a normalMinimum of 0.0</assert>
      <assert test="@normalMaximum castable as xs:float and xs:float(@normalMaximum) eq 1.0"
        >outcomeDeclaration "SCORE" must have a normalMaximum of 1.0</assert>
      <assert test="empty(qti:defaultValue)">outcomeDeclaration SCORE is not allowed to contain a
        defaulValue</assert>
    </rule>
    <!-- -->
    <rule context="qti:outcomeDeclaration[$ItemWithInteraction][@identifier eq 'FEEDBACK']">
      <assert test="@baseType eq 'identifier'">outcomeDeclaration "FEEDBACK" must have baseType
        "identifier"</assert>
      <assert test="empty(qti:defaultValue)">outcomeDeclaration FEEDBACK is not allowed to contain a
        defaulValue</assert>
      <assert test="empty(@normalMinimum) and empty(@normalMaximum)">outcomeDeclaration FEEDBACK is
        not allowed to contain normalMinimum or normalMaximum attribute</assert>
    </rule>
    <!-- -->
    <rule
      context="qti:outcomeDeclaration[$ItemWithInteraction][@identifier eq 'FEEDBACK_THRESHOLD']">
      <assert test="@baseType eq 'float'">outcomeDeclaration "FEEDBACK_THRESHOLD" must have baseType
        "float"</assert>
      <assert test="qti:defaultValue/qti:value[@baseType eq 'float'][. castable as xs:float]"
        >outcomeDeclaration "FEEDBACK_THRESHOLD" must contain a default value of type
        "float"</assert>
      <assert
        test="(xs:float(qti:defaultValue/qti:value) ge 0.0) and (xs:float(qti:defaultValue/qti:value) le 1.0)"
        >outcomeDeclaration "FEEDBACK_THRESHOLD" must have a default value in between 0.0 and
        1.0</assert>
      <assert test="empty(@normalMinimum) and empty(@normalMaximum)">outcomeDeclaration
        FEEDBACK_THRESHOLD is not allowed to contain a normalMinimum of normalMaximum
        attribute</assert>
    </rule>
    <!-- -->
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- BASIC CHECKS FOR THE VARIOUS INTERACTION TYPES: -->
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'associateInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'choiceInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'extendedTextInteraction'"/>
    <param name="HasCorrectResponse" value="false()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'graphicGapMatchInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'gapMatchInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'hotspotInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'hottextInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'matchInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'orderInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'selectPointInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckSingleInteractionOccurence">
    <param name="InteractionType" value="'sliderInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckMultipleInteractionOccurence">
    <param name="InteractionType" value="'inlineChoiceInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <!-- mediaInteraction is not separately checked. -->
  <!-- -->
  <pattern is-a="CheckMultipleInteractionOccurence">
    <param name="InteractionType" value="'positionObjectInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <pattern is-a="CheckMultipleInteractionOccurence">
    <param name="InteractionType" value="'textEntryInteraction'"/>
    <param name="HasCorrectResponse" value="true()"/>
  </pattern>
  <!-- -->
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <!-- Abstract pattern that checks the basics for a single occurence interaction type: -->
  <!-- -->
  <pattern abstract="true" id="CheckSingleInteractionOccurence">
    <!-- -->
    <rule
      context="qti:itemBody[$ItemWithInteraction]
        [exists(.//qti:*[local-name() eq $InteractionType])]">
      <!-- We may have only one: -->
      <assert test="$InteractionCountWithoutMedia eq 1">Interaction type <value-of
          select="$InteractionType"/>: Only one interaction in the itemBody allowed</assert>
    </rule>
    <!-- -->
    <rule context="qti:itemBody[$ItemWithInteraction]//qti:*[local-name() eq $InteractionType]">
      <!-- It must refer to the right responseDeclaration called RESPONSE: -->
      <assert test="@responseIdentifier eq 'RESPONSE'">Interaction type <value-of
          select="$InteractionType"/>: responseIdentifier must have the identifier
        "RESPONSE"</assert>
    </rule>
    <!-- -->
    <rule
      context="/qti:assessmentItem[$ItemWithInteraction][exists(qti:itemBody//qti:*[local-name() eq $InteractionType])]">
      <!-- If we have must have a correct response, we also need a score: -->
      <assert test="not($HasCorrectResponse) or qti:outcomeDeclaration[@identifier eq 'SCORE']"
        >Interaction type <value-of select="$InteractionType"/>: No outcomeDeclaration "SCORE"
        present</assert>
      <!-- So there must also be only one responseDeclaration (besides media ones): -->
      <assert test="count(qti:responseDeclaration[not(starts-with(@identifier, 'MEDIA_'))]) eq 1"
        >Interaction type <value-of select="$InteractionType"/>: More than one responseDeclaration
        element present</assert>
      <!-- This must be called RESPONSE: -->
      <assert test="exists(qti:responseDeclaration[@identifier eq 'RESPONSE'])">Interaction type
          <value-of select="$InteractionType"/>: responseDeclaration must have the identifier
        "RESPONSE"</assert>
      <!-- If a correctResponse is necessary, check if it exists: -->
      <assert
        test="not($HasCorrectResponse) or not(exists(qti:responseDeclaration[@identifier eq 'RESPONSE'])) or
              exists(qti:responseDeclaration[@identifier eq 'RESPONSE']/qti:correctResponse)"
        >Interaction type <value-of select="$InteractionType"/>: responseDeclaration "RESPONSE" must
        contain a correctResponse element</assert>
      <!-- If a correctResponse is not necessary, check if it is absent: -->
      <assert
        test="$HasCorrectResponse or not(exists(qti:responseDeclaration[@identifier eq 'RESPONSE'])) or
        not(exists(qti:responseDeclaration[@identifier eq 'RESPONSE']/qti:correctResponse))"
        >Interaction type <value-of select="$InteractionType"/>: responseDeclaration "RESPONSE"
        should not contain a correctResponse element</assert>
    </rule>
    <!-- -->
    <!-- Perform some checks on the response processing: -->
    <rule
      context="/qti:assessmentItem[$ItemWithInteraction][exists(qti:itemBody//qti:*[local-name() eq $InteractionType])]
      /qti:responseProcessing">
      <!-- There must be a template attribute (values are checked in the schema): -->
      <assert test="exists(@template)">Interaction type <value-of select="$InteractionType"/>:
        responseProcessing must contain a template attribute</assert>
      <!-- There must be a templateLocation attribute: -->
      <assert test="exists(@templateLocation)">Interaction type <value-of select="$InteractionType"
        />: responseProcessing must contain a templateLocation attribute</assert>
      <!-- There can be no content: -->
      <assert test="not(exists(*))">Interaction type <value-of select="$InteractionType"/>:
        responseProcessing must be empty</assert>
    </rule>
    <!-- -->
  </pattern>
  <!-- -->
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <!-- Abstract pattern that checks the basics for a multiple occurence interaction type: -->
  <!-- -->
  <pattern abstract="true" id="CheckMultipleInteractionOccurence">
    <!-- -->
    <rule
      context="/qti:assessmentItem[$ItemWithInteraction]/qti:itemBody//qti:*[local-name() eq $InteractionType]">
      <let name="ResponseIdentifier" value="@responseIdentifier"/>
      <!-- Is there a responseDeclaration with the corresponsing identifier? -->
      <assert test="exists(/*/qti:responseDeclaration[@identifier eq $ResponseIdentifier])"
        >Interaction type <value-of select="$InteractionType"/>: No responseDeclaration element with
        identifier "<value-of select="$ResponseIdentifier"/>" found</assert>
    </rule>
    <!-- -->
    <rule
      context="/qti:assessmentItem[$ItemWithInteraction]
        [qti:itemBody//qti:*[local-name() eq $InteractionType]]
        /qti:responseDeclaration[starts-with(@identifier, 'RESPONSE_')]">
      <!-- A responseDeclaration must have a correctResponse (only when a correct response is mandatory): -->
      <assert test="not($HasCorrectResponse) or exists(qti:correctResponse)">Interaction type
          <value-of select="$InteractionType"/>: responseDeclaration "<value-of select="@identifier"
        />" must contain a correctResponse element</assert>
    </rule>
    <!-- -->
    <!-- Perform some checks on the response processing: -->
    <rule
      context="/qti:assessmentItem[$ItemWithInteraction][exists(qti:itemBody//qti:*[local-name() eq $InteractionType])]
      /qti:responseProcessing">
      <!-- There must be content or a template: -->
      <assert test="not(exists(*) and exists(@template))">Interaction type <value-of
          select="$InteractionType"/>: Using both direct responseProcessing and a template attribute
        is not allowed</assert>
      <!-- -->
      <!-- If you use a template, the responseDeclaration identifiers must be RESPONSE_xx: -->
      <assert
        test="not(exists(@template)) or 
          (every $rp in /*/qti:responseDeclaration satisfies matches($rp/@identifier, '^RESPONSE_[0-9]{2}$'))"
        >Interaction type <value-of select="$InteractionType"/>: responseDeclaration identifiers are
        not valid for use with a response processing template responseProcessing template: Use
        RESPONSE_xx (xx is a two digit number)</assert>
    </rule>
    <!-- -->
    <!-- Check for consistent use of mapping/areaMapping elements within responseDeclaration's: -->
    <rule
      context="/qti:assessmentItem[$ItemWithInteraction][exists(qti:itemBody//qti:*[local-name() eq $InteractionType])]
        [exists(qti:responseDeclaration/qti:mapping)]">
      <assert test="every $rd in qti:responseDeclaration satisfies exists($rd/qti:mapping)"
        >Interaction type <value-of select="$InteractionType"/>: Inconsistent usage of mapping
        elements in the responseDeclaration elements (use mapping in all responseDeclaration
        elements or nowhere)</assert>
    </rule>
    <rule
      context="/qti:assessmentItem[$ItemWithInteraction][exists(qti:itemBody//qti:*[local-name() eq $InteractionType])]
        [exists(qti:responseDeclaration/qti:areaMapping)]">
      <assert test="every $rd in qti:responseDeclaration satisfies exists($rd/qti:areaMapping)"
        >Interaction type <value-of select="$InteractionType"/>: Inconsistent usage of areaMapping
        elements in the responseDeclaration elements (use areaMapping in all responseDeclaration
        elements or nowhere)</assert>
    </rule>
    <!-- -->
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- DETAILED CHECKS FOR SOME INTERACTION TYPES: -->
  <!-- -->
  <pattern>
    <!-- -->
    <!-- extendedTextInteraction: -->
    <rule context="qti:itemBody[$ItemWithInteraction]//qti:extendedTextInteraction">
      <!-- Remark: This could have been checked by the schema. However, extendedText and textEntry 
        share the same attribute group for this and textEntry does need this attribute. -->
      <assert test="not(exists(@patternMask))">Interaction type extendedTextInteraction: Attribute
        patternMask not allowed for extended text</assert>
    </rule>
  </pattern>
  <!-- -->
</schema>
