<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron" queryBinding="xslt2">
  <!-- ================================================================== -->
  <!--* ISO Schematron to verify QTI tests according to the NLQTI profile. -->
  <!--
    For full validation: Validate first against the (adapted) schema and after that against this 
    file.  
  
    SVN: $Id: imsqti_v2p1_NLQTI_v1p1_tests.sch 1317 2013-08-08 11:56:56Z erik $ 
  -->
  <!-- ================================================================== -->
  <!-- -->
  <ns uri="http://www.imsglobal.org/xsd/imsqti_v2p1" prefix="qti"/>
  <ns uri="http://www.w3.org/2001/XMLSchema" prefix="xs"/>
  <!-- -->
  <let name="HasFeedback" value="exists(/qti:assessmentTest/qti:testFeedback)"/>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECK THE OVERALL STRUCTURE: -->
  <!-- -->
  <pattern>
    <!-- -->
    <rule context="/qti:assessmentTest">
      <assert
        test="every $od in qti:outcomeDeclaration satisfies $od/@identifier = ('SCORE', 'FEEDBACK', 'FEEDBACK_THRESHOLD')"
        >outcomeDeclaration element(s) found with illegal identifier (allowed: "SCORE", "FEEDBACK",
        "FEEDBACK_THRESHOLD")</assert>
    </rule>
    <!-- -->
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECKS FOR FEEDBACKS: -->
  <!-- -->
  <pattern>
    <!-- -->
    <rule context="/qti:assessmentTest[$HasFeedback]">
      <assert test="qti:outcomeDeclaration[@identifier eq 'FEEDBACK']">Item with feedback found
        without outcomeDeclaration "FEEDBACK"</assert>
      <assert test="qti:outcomeDeclaration[@identifier eq 'FEEDBACK_THRESHOLD']">Item with feedback
        found without outcomeDeclaration "FEEDBACK_THRESHOLD"</assert>
      <assert
        test="every $mf in qti:testFeedback satisfies $mf/@identifier = ('RESULT_OK', 'RESULT_NOTOK')"
        >testFeedback element(s) found with invalid identifier (allowed: "RESULT_OK",
        "RESULT_NOTOK")</assert>
      <assert test="qti:testFeedback[@identifier eq 'RESULT_NOTOK']">testFeedback "RESULT_NOTOK" not
        found</assert>
      <assert
        test="not(qti:testFeedback[@identifier eq 'RESULT_OK']) or 
          (qti:testFeedback[@identifier eq 'RESULT_OK'] and qti:testFeedback[@identifier eq 'RESULT_NOTOK'])"
        >testFeedback "RESULT_OK" found without a testFeedback "RESULT_NOTOK"</assert>
      <!-- -->
      <assert test="count(/qti:assessmentTest/qti:outcomeProcessing/qti:outcomeCondition) eq 2">A test with feedback
        must contain two outcomeProcessing/outcomeCondition elements</assert>
    </rule>
    <!-- -->
    <rule context="qti:assessmentItem[not($HasFeedback)]">
      <assert test="not(qti:outcomeDeclaration[@identifier eq 'FEEDBACK'])">outcomeDeclaration
        "FEEDBACK" found for an item without feedback</assert>
      <assert test="not(qti:outcomeDeclaration[@identifier eq 'FEEDBACK_THRESHOLD'])"
        >outcomeDeclaration "FEEDBACK_THRESHOLD" found for an item without feedback</assert>
      <!-- -->
      <assert test="count(/qti:assessmentTest/qti:outcomeProcessing/qti:outcomeCondition) eq 1">A test without
        feedback must only contain a single outcomeProcessing/outcomeCondition element</assert>
    </rule>
    <!-- -->
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECK DETAILS FOR THE OUTCOME DECLARATIONS: -->
  <!-- -->
  <pattern>
    <!-- -->
    <rule context="qti:outcomeDeclaration[@identifier eq 'SCORE']">
      <assert test="@baseType eq 'float'">outcomeDeclaration "SCORE" must have baseType
        "float"</assert>
      <assert test="@normalMinimum castable as xs:float and xs:float(@normalMinimum) eq 0.0"
        >outcomeDeclaration "SCORE" must have a normalMinimum of 0.0</assert>
      <assert test="@normalMaximum castable as xs:float and xs:float(@normalMaximum) eq 1.0"
        >outcomeDeclaration "SCORE" must have a normalMaximum of 1.0</assert>
      <assert test="empty(qti:defaultValue)">outcomeDeclaration SCORE must not contain a
        defaulValue</assert>
    </rule>
    <!-- -->
    <rule context="qti:outcomeDeclaration[@identifier eq 'FEEDBACK']">
      <assert test="@baseType eq 'identifier'">outcomeDeclaration "FEEDBACK" must have baseType
        "identifier"</assert>
      <assert test="empty(qti:defaultValue)">outcomeDeclaration FEEDBACK must not contain a
        defaulValue</assert>
      <assert test="empty(@normalMinimum) and empty(@normalMaximum)">outcomeDeclaration FEEDBACK
        must not contain a normalMinimum or normalMaximum attribute</assert>
    </rule>
    <!-- -->
    <rule context="qti:outcomeDeclaration[@identifier eq 'FEEDBACK_THRESHOLD']">
      <assert test="@baseType eq 'float'">outcomeDeclaration "FEEDBACK_THRESHOLD" must have baseType
        "float"</assert>
      <assert test="qti:defaultValue/qti:value[@baseType eq 'float'][. castable as xs:float]"
        >outcomeDeclaration "FEEDBACK_THRESHOLD" must have a default value of type "float"</assert>
      <assert
        test="(xs:float(qti:defaultValue/qti:value) ge 0.0) and (xs:float(qti:defaultValue/qti:value) le 1.0)"
        >outcomeDeclaration "FEEDBACK_THRESHOLD"must have a default value in between 0.0 and
        1.0</assert>
      <assert test="empty(@normalMinimum) and empty(@normalMaximum)">outcomeDeclaration
        FEEDBACK_THRESHOLD is not allowed to contain a normalMinimum of normalMaximum
        attribute</assert>
    </rule>
    <!-- -->
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECKS FOR ALL ASSESSSMENTSECTIONS: -->
  <!-- -->
  <pattern>
    <rule context="qti:assessmentSection/qti:selection">
      <assert
        test="xs:integer(@select) le count((../qti:assessmentSection, ../qti:assessmentItemRef))"
        >The value of the select attribute on a selection element must be less than or equal to the
        total number of assessmentSection + assessmentItemRef elements</assert>
    </rule>
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECKS ON MAIN ASSESSMENTSECTION LEVEL: -->
  <!-- -->
  <pattern>
    <rule context="qti:assessmentSection[parent::qti:testPart]">
      <assert test="empty(@required)">A required attribute is not allowed at the main
        assessmentSection level</assert>
      <assert test="empty(@keepTogether)">A keepTogether attribute is not allowed at the main
        assessmentSection level</assert>
      <assert test="@visible eq 'true'">The visible attribute must have a value of true the main
        assessmentSection level</assert>
    </rule>
  </pattern>
  <!-- -->
  <!-- ================================================================== -->
  <!-- CHECKS ON SUBASSESSMENTSECTION LEVEL: -->
  <!-- -->
  <pattern>
    <rule context="qti:assessmentSection[parent::qti:assessmentSection]">
      <assert test="empty(qti:assessmentSection)">At the sub assessmentSection level, no further
        assessmentSection's are allowed</assert>
    </rule>
  </pattern>
  <!-- -->
</schema>
